#include <Bela.h>
#include <Scope.h>
#include <stdlib.h>
#include <math.h>
#include "inductorwah.h"

#define EXP_PEDAL_CHANNEL 7
#define CF_CUTOFF 8 //Hz, analog channels filtering cutoff
#define SCAN_PERIOD 50    //ms (used for int).  Time between evaluating whether a control changed

//channel_filter stores values from analog inputs.  Filtering is run on these inputs 
// and stored in the filtered_reading array.
// Also includes state variables for detecting whether there was a position change 
// then a timer to set how long after detecting initial change that it will continue
// to set the value to the active effect defined by control context
// Also maintains state variables for switch function detection.
typedef struct channel_filter_t {
	
	//Store values read from the analog inputs
	float current_reading;
	float filtered_reading[2];
	
	//Evaluate whether a control position has changed
	unsigned int scan_cycle;
	unsigned int scan_timer;
	unsigned int active_timer;
	unsigned int active_reset;
	float last_reading;
	float dVdt;
	
	//first order filter coefficients.
	float a;  
	float b;
	float c;
	float d;
	
	//switch function
	bool switch_state;
	bool edge;
	unsigned int knob_toggle_timer;

} channel_filter;

int gAudioFramesPerAnalogFrame;

channel_filter knobs;
iwah_coeffs* iwah;

Scope scope;

bool setup(BelaContext *context, void *userData)
{
	//Setup scope
	scope.setup(2, context->audioSampleRate);
	//Setup wah wah 
	iwah = make_iwah(iwah, context->audioSampleRate);
	iwah_circuit_preset(VOX, iwah, context->audioSampleRate);
	
	gAudioFramesPerAnalogFrame = context->audioFrames / context->analogFrames;
	float nframes = context->audioFrames;
	float gifs = 1.0/context->audioSampleRate;
	float dpfc = 2.0*M_PI*CF_CUTOFF*gifs;

	knobs.current_reading = 0.25;
	knobs.filtered_reading[0] = 0.5;
	knobs.filtered_reading[1] = 0.5;
	knobs.a = dpfc/(dpfc + 1.0);
	knobs.b = 1.0 - knobs.a;
	knobs.c = 0.5*dpfc/(dpfc + 1.0);
	knobs.d = 1.0 - knobs.c;
	
	//Knob switch functions
	knobs.switch_state = true;
	knobs.edge = false;
	knobs.knob_toggle_timer = 0;

	//State change detection
	knobs.active_timer = 0;
	knobs.active_reset = (int)(2*context->audioSampleRate)/nframes;
	knobs.scan_cycle = ((int) context->audioSampleRate) * SCAN_PERIOD / (1000 * nframes);
	knobs.scan_timer = 0;
	knobs.last_reading = 0.5;
	knobs.dVdt = 0.01;
	
	return true;
}

void render(BelaContext *context, void *userData)
{
	float tmp = 0.0;
	float ch0 = 0.0;

	for(unsigned int n = 0; n < context->audioFrames; n++) 
		{
			if(!(n % gAudioFramesPerAnalogFrame)) {
					
				tmp = analogRead(context, n/gAudioFramesPerAnalogFrame, EXP_PEDAL_CHANNEL);
				knobs.current_reading = tmp;
	
			}
			//Filter the analog channel readings
			knobs.filtered_reading[0] = knobs.a*knobs.current_reading + knobs.b*knobs.filtered_reading[0];		
			knobs.filtered_reading[1] = knobs.c*knobs.filtered_reading[0] + knobs.d*knobs.filtered_reading[1];
			
			//Run wah 
			float dry = audioRead(context, n, 0);
			ch0 = iwah_tick(dry, knobs.filtered_reading[1], iwah);
			scope.log(dry,ch0);
			audioWrite(context, n, 0, ch0);
	
		}  

}

void cleanup(BelaContext *context, void *userData)
{
	free(iwah);
}